from unittest.mock import AsyncMock

import pytest
from bleak.backends.device import BLEDevice

from switchbot import SwitchBotAdvertisement, SwitchbotModel
from switchbot.devices import relay_switch

from .test_adv_parser import generate_ble_device


def create_device_for_command_testing(calibration=True, reverse_mode=False):
    ble_device = generate_ble_device("aa:bb:cc:dd:ee:ff", "any")
    relay_switch_device = relay_switch.SwitchbotRelaySwitch(
        ble_device, "ff", "ffffffffffffffffffffffffffffffff"
    )
    relay_switch_device.update_from_advertisement(make_advertisement_data(ble_device))
    return relay_switch_device


def make_advertisement_data(ble_device: BLEDevice):
    """Set advertisement data with defaults."""
    return SwitchBotAdvertisement(
        address="aa:bb:cc:dd:ee:ff",
        data={
            "rawAdvData": b"$X|\x0866G\x81\x00\x00\x001\x00\x00\x00\x00",
            "data": {
                "switchMode": True,
                "sequence_number": 71,
                "isOn": True,
                "power": 4.9,
                "voltage": 0,
                "current": 0,
            },
            "isEncrypted": False,
            "model": "<",
            "modelFriendlyName": "Relay Switch 1PM",
            "modelName": SwitchbotModel.RELAY_SWITCH_1PM,
        },
        device=ble_device,
        rssi=-80,
        active=True,
    )


@pytest.mark.asyncio
async def test_turn_on():
    relay_switch_device = create_device_for_command_testing()
    relay_switch_device._send_command = AsyncMock(return_value=b"\x01")
    await relay_switch_device.turn_on()
    assert relay_switch_device.is_on() is True


@pytest.mark.asyncio
async def test_turn_off():
    relay_switch_device = create_device_for_command_testing()
    relay_switch_device._send_command = AsyncMock(return_value=b"\x01")
    await relay_switch_device.turn_off()
    assert relay_switch_device.is_on() is False


@pytest.mark.asyncio
async def test_get_basic_info():
    relay_switch_device = create_device_for_command_testing()
    relay_switch_device._send_command = AsyncMock(return_value=b"\x01\x01")
    info = await relay_switch_device.get_basic_info()
    assert info["is_on"] is True
    relay_switch_device._send_command = AsyncMock(return_value=b"\x01\x00")
    info = await relay_switch_device.get_basic_info()
    assert info["is_on"] is False
    relay_switch_device._send_command = AsyncMock(return_value=b"\x00\x00")
    info = await relay_switch_device.get_basic_info()
    assert info is None



from __future__ import annotations

import asyncio
from collections.abc import Coroutine
from typing import Any, TypeVar

_R = TypeVar("_R")

_BACKGROUND_TASKS: set[asyncio.Task[Any]] = set()


def create_background_task(target: Coroutine[Any, Any, _R]) -> asyncio.Task[_R]:
    """Create a background task."""
    task = asyncio.create_task(target)
    _BACKGROUND_TASKS.add(task)
    task.add_done_callback(_BACKGROUND_TASKS.remove)
    return task



import datetime
from unittest.mock import AsyncMock

import pytest
from bleak.backends.device import BLEDevice

from switchbot import SwitchBotAdvertisement, SwitchbotModel
from switchbot.adv_parsers.humidifier import process_evaporative_humidifier
from switchbot.const.evaporative_humidifier import HumidifierMode, HumidifierWaterLevel
from switchbot.devices import evaporative_humidifier

from .test_adv_parser import generate_ble_device


def create_device_for_command_testing(init_data: dict | None = None):
    ble_device = generate_ble_device("aa:bb:cc:dd:ee:ff", "any")
    evaporative_humidifier_device = (
        evaporative_humidifier.SwitchbotEvaporativeHumidifier(
            ble_device, "ff", "ffffffffffffffffffffffffffffffff"
        )
    )
    evaporative_humidifier_device.update_from_advertisement(
        make_advertisement_data(ble_device, init_data)
    )
    return evaporative_humidifier_device


def make_advertisement_data(ble_device: BLEDevice, init_data: dict | None = None):
    if init_data is None:
        init_data = {}
    """Set advertisement data with defaults."""
    return SwitchBotAdvertisement(
        address="aa:bb:cc:dd:ee:ff",
        data={
            "rawAdvData": b"#\x00\x00\x15\x1c\x00",
            "data": {
                "isOn": False,
                "mode": None,
                "target_humidity": None,
                "child_lock": False,
                "over_humidify_protection": True,
                "tank_removed": False,
                "tilted_alert": False,
                "filter_missing": False,
                "humidity": 51,
                "temperature": 16.8,
                "filter_run_time": datetime.timedelta(days=3, seconds=57600),
                "filter_alert": False,
                "water_level": HumidifierWaterLevel.LOW,
            }
            | init_data,
            "isEncrypted": False,
            "model": "#",
            "modelFriendlyName": "Evaporative Humidifier",
            "modelName": SwitchbotModel.EVAPORATIVE_HUMIDIFIER,
        },
        device=ble_device,
        rssi=-80,
        active=True,
    )


@pytest.mark.asyncio
async def test_process_advertisement():
    data = process_evaporative_humidifier(
        b"#\x00\x00\x15\x1c\x00",
        b"\xd4\x8cIU\x95\xb2\x08\x06\x88\xb3\x90\x81\x00X\x00X2",
    )

    assert data == {
        "isOn": False,
        "mode": None,
        "target_humidity": None,
        "child_lock": False,
        "over_humidify_protection": None,
        "tank_removed": False,
        "tilted_alert": False,
        "filter_missing": False,
        "humidity": 51,
        "temperature": 16.8,
        "filter_run_time": datetime.timedelta(days=3, seconds=57600),
        "filter_alert": False,
        "water_level": HumidifierWaterLevel.LOW,
    }


@pytest.mark.asyncio
async def test_process_advertisement_empty():
    data = process_evaporative_humidifier(None, None)

    assert data == {
        "isOn": None,
        "mode": None,
        "target_humidity": None,
        "child_lock": None,
        "over_humidify_protection": None,
        "tank_removed": None,
        "tilted_alert": None,
        "filter_missing": None,
        "humidity": None,
        "temperature": None,
        "filter_run_time": None,
        "filter_alert": None,
        "water_level": None,
    }


@pytest.mark.asyncio
async def test_turn_on():
    device = create_device_for_command_testing({"isOn": False})
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.is_on() is False
    await device.turn_on()
    assert device.is_on() is True


@pytest.mark.asyncio
async def test_turn_off():
    device = create_device_for_command_testing({"isOn": True})
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.is_on() is True
    await device.turn_off()
    assert device.is_on() is False


@pytest.mark.asyncio
async def test_set_mode():
    device = create_device_for_command_testing(
        {"isOn": True, "mode": HumidifierMode.LOW}
    )
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.get_mode() is HumidifierMode.LOW
    await device.set_mode(HumidifierMode.AUTO)
    assert device.get_mode() is HumidifierMode.AUTO

    await device.set_mode(HumidifierMode.TARGET_HUMIDITY, 60)
    assert device.get_mode() is HumidifierMode.TARGET_HUMIDITY
    assert device.get_target_humidity() == 60

    await device.set_mode(HumidifierMode.DRYING_FILTER)
    assert device.get_mode() is HumidifierMode.DRYING_FILTER

    with pytest.raises(ValueError):  # noqa: PT011
        await device.set_mode(0)

    with pytest.raises(TypeError):
        await device.set_mode(HumidifierMode.TARGET_HUMIDITY)


@pytest.mark.asyncio
async def test_set_child_lock():
    device = create_device_for_command_testing({"child_lock": False})
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.is_child_lock_enabled() is False
    await device.set_child_lock(True)
    assert device.is_child_lock_enabled() is True


@pytest.mark.asyncio
async def test_start_drying_filter():
    device = create_device_for_command_testing(
        {"isOn": True, "mode": HumidifierMode.AUTO}
    )
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.get_mode() is HumidifierMode.AUTO
    await device.start_drying_filter()
    assert device.get_mode() is HumidifierMode.DRYING_FILTER


@pytest.mark.asyncio
async def test_stop_drying_filter():
    device = create_device_for_command_testing(
        {"isOn": True, "mode": HumidifierMode.DRYING_FILTER}
    )
    device._send_command = AsyncMock(return_value=b"\x00")

    assert device.is_on() is True
    assert device.get_mode() is HumidifierMode.DRYING_FILTER
    await device.stop_drying_filter()
    assert device.is_on() is False
    assert device.get_mode() is None


@pytest.mark.asyncio
async def test_attributes():
    device = create_device_for_command_testing()
    device._send_command = AsyncMock(return_value=b"\x01")

    assert device.is_over_humidify_protection_enabled() is True
    assert device.is_tank_removed() is False
    assert device.is_filter_missing() is False
    assert device.is_filter_alert_on() is False
    assert device.is_tilted_alert_on() is False
    assert device.get_water_level() is HumidifierWaterLevel.LOW
    assert device.get_filter_run_time() == datetime.timedelta(days=3, seconds=57600)
    assert device.get_humidity() == 51
    assert device.get_temperature() == 16.8



import logging
from typing import Any

from bleak.backends.device import BLEDevice

from ..const import SwitchbotModel
from ..const.evaporative_humidifier import (
    TARGET_HUMIDITY_MODES,
    HumidifierMode,
    HumidifierWaterLevel,
)
from ..models import SwitchBotAdvertisement
from .device import SwitchbotEncryptedDevice

_LOGGER = logging.getLogger(__name__)

COMMAND_HEADER = "57"
COMMAND_GET_CK_IV = f"{COMMAND_HEADER}0f2103"
COMMAND_TURN_ON = f"{COMMAND_HEADER}0f430101"
COMMAND_TURN_OFF = f"{COMMAND_HEADER}0f430100"
COMMAND_CHILD_LOCK_ON = f"{COMMAND_HEADER}0f430501"
COMMAND_CHILD_LOCK_OFF = f"{COMMAND_HEADER}0f430500"
COMMAND_AUTO_DRY_ON = f"{COMMAND_HEADER}0f430a01"
COMMAND_AUTO_DRY_OFF = f"{COMMAND_HEADER}0f430a02"
COMMAND_SET_MODE = f"{COMMAND_HEADER}0f4302"
COMMAND_GET_BASIC_INFO = f"{COMMAND_HEADER}000300"

MODES_COMMANDS = {
    HumidifierMode.HIGH: "010100",
    HumidifierMode.MEDIUM: "010200",
    HumidifierMode.LOW: "010300",
    HumidifierMode.QUIET: "010400",
    HumidifierMode.TARGET_HUMIDITY: "0200",
    HumidifierMode.SLEEP: "0300",
    HumidifierMode.AUTO: "040000",
}


class SwitchbotEvaporativeHumidifier(SwitchbotEncryptedDevice):
    """Representation of a Switchbot Evaporative Humidifier"""

    def __init__(
        self,
        device: BLEDevice,
        key_id: str,
        encryption_key: str,
        interface: int = 0,
        model: SwitchbotModel = SwitchbotModel.EVAPORATIVE_HUMIDIFIER,
        **kwargs: Any,
    ) -> None:
        self._force_next_update = False
        super().__init__(device, key_id, encryption_key, model, interface, **kwargs)

    @classmethod
    async def verify_encryption_key(
        cls,
        device: BLEDevice,
        key_id: str,
        encryption_key: str,
        model: SwitchbotModel = SwitchbotModel.EVAPORATIVE_HUMIDIFIER,
        **kwargs: Any,
    ) -> bool:
        return await super().verify_encryption_key(
            device, key_id, encryption_key, model, **kwargs
        )

    def update_from_advertisement(self, advertisement: SwitchBotAdvertisement) -> None:
        """Update device data from advertisement."""
        super().update_from_advertisement(advertisement)
        _LOGGER.debug(
            "%s: update advertisement: %s",
            self.name,
            advertisement,
        )

    async def _get_basic_info(self) -> bytes | None:
        """Return basic info of device."""
        _data = await self._send_command(
            key=COMMAND_GET_BASIC_INFO, retry=self._retry_count
        )

        if _data in (b"\x07", b"\x00"):
            _LOGGER.error("Unsuccessful, please try again")
            return None

        return _data

    async def get_basic_info(self) -> dict[str, Any] | None:
        """Get device basic settings."""
        if not (_data := await self._get_basic_info()):
            return None

        # Not 100% sure about this data, will verify once a firmware update is available
        return {
            "firmware": _data[2] / 10.0,
        }

    async def turn_on(self) -> bool:
        """Turn device on."""
        result = await self._send_command(COMMAND_TURN_ON)
        if ok := self._check_command_result(result, 0, {1}):
            self._override_state({"isOn": True})
            self._fire_callbacks()
        return ok

    async def turn_off(self) -> bool:
        """Turn device off."""
        result = await self._send_command(COMMAND_TURN_OFF)
        if ok := self._check_command_result(result, 0, {1}):
            self._override_state({"isOn": False})
            self._fire_callbacks()
        return ok

    async def set_mode(
        self, mode: HumidifierMode, target_humidity: int | None = None
    ) -> bool:
        """Set device mode."""
        if mode == HumidifierMode.DRYING_FILTER:
            return await self.start_drying_filter()
        if mode not in MODES_COMMANDS:
            raise ValueError("Invalid mode")

        command = COMMAND_SET_MODE + MODES_COMMANDS[mode]
        if mode in TARGET_HUMIDITY_MODES:
            if target_humidity is None:
                raise TypeError("target_humidity is required")
            command += f"{target_humidity:02x}"
        result = await self._send_command(command)
        if ok := self._check_command_result(result, 0, {1}):
            self._override_state({"mode": mode})
            if mode == HumidifierMode.TARGET_HUMIDITY and target_humidity is not None:
                self._override_state({"target_humidity": target_humidity})
            self._fire_callbacks()
        return ok

    async def set_child_lock(self, enabled: bool) -> bool:
        """Set child lock."""
        result = await self._send_command(
            COMMAND_CHILD_LOCK_ON if enabled else COMMAND_CHILD_LOCK_OFF
        )
        if ok := self._check_command_result(result, 0, {1}):
            self._override_state({"child_lock": enabled})
            self._fire_callbacks()
        return ok

    async def start_drying_filter(self):
        """Start drying filter."""
        result = await self._send_command(COMMAND_TURN_ON + "08")
        if ok := self._check_command_result(result, 0, {1}):
            self._override_state({"mode": HumidifierMode.DRYING_FILTER})
            self._fire_callbacks()
        return ok

    async def stop_drying_filter(self):
        """Stop drying filter."""
        result = await self._send_command(COMMAND_TURN_OFF)
        if ok := self._check_command_result(result, 0, {0}):
            self._override_state({"isOn": False, "mode": None})
            self._fire_callbacks()
        return ok

    def is_on(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("isOn")

    def get_mode(self) -> HumidifierMode | None:
        """Return state from cache."""
        return self._get_adv_value("mode")

    def is_child_lock_enabled(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("child_lock")

    def is_over_humidify_protection_enabled(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("over_humidify_protection")

    def is_tank_removed(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("tank_removed")

    def is_filter_missing(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("filter_missing")

    def is_filter_alert_on(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("filter_alert")

    def is_tilted_alert_on(self) -> bool | None:
        """Return state from cache."""
        return self._get_adv_value("tilted_alert")

    def get_water_level(self) -> HumidifierWaterLevel | None:
        """Return state from cache."""
        return self._get_adv_value("water_level")

    def get_filter_run_time(self) -> int | None:
        """Return state from cache."""
        return self._get_adv_value("filter_run_time")

    def get_target_humidity(self) -> int | None:
        """Return state from cache."""
        return self._get_adv_value("target_humidity")

    def get_humidity(self) -> int | None:
        """Return state from cache."""
        return self._get_adv_value("humidity")

    def get_temperature(self) -> float | None:
        """Return state from cache."""
        return self._get_adv_value("temperature")



"""Relay Switch adv parser."""

from __future__ import annotations


def process_worelay_switch_1pm(
    data: bytes | None, mfr_data: bytes | None
) -> dict[str, bool | int]:
    """Process WoStrip services data."""
    if mfr_data is None:
        return {}
    return {
        "switchMode": True,  # for compatibility, useless
        "sequence_number": mfr_data[6],
        "isOn": bool(mfr_data[7] & 0b10000000),
        "power": ((mfr_data[10] << 8) + mfr_data[11]) / 10,
        "voltage": 0,
        "current": 0,
    }


def process_worelay_switch_1(
    data: bytes | None, mfr_data: bytes | None
) -> dict[str, bool | int]:
    """Process WoStrip services data."""
    if mfr_data is None:
        return {}
    return {
        "switchMode": True,  # for compatibility, useless
        "sequence_number": mfr_data[6],
        "isOn": bool(mfr_data[7] & 0b10000000),
    }



import logging
import time
from typing import Any

from bleak.backends.device import BLEDevice

from ..const import SwitchbotModel
from ..models import SwitchBotAdvertisement
from .device import SwitchbotEncryptedDevice

_LOGGER = logging.getLogger(__name__)

COMMAND_HEADER = "57"
COMMAND_TURN_OFF = f"{COMMAND_HEADER}0f70010000"
COMMAND_TURN_ON = f"{COMMAND_HEADER}0f70010100"
COMMAND_TOGGLE = f"{COMMAND_HEADER}0f70010200"
COMMAND_GET_VOLTAGE_AND_CURRENT = f"{COMMAND_HEADER}0f7106000000"
COMMAND_GET_SWITCH_STATE = f"{COMMAND_HEADER}0f7101000000"
PASSIVE_POLL_INTERVAL = 10 * 60


class SwitchbotRelaySwitch(SwitchbotEncryptedDevice):
    """Representation of a Switchbot relay switch 1pm."""

    def __init__(
        self,
        device: BLEDevice,
        key_id: str,
        encryption_key: str,
        interface: int = 0,
        model: SwitchbotModel = SwitchbotModel.RELAY_SWITCH_1PM,
        **kwargs: Any,
    ) -> None:
        self._force_next_update = False
        super().__init__(device, key_id, encryption_key, model, interface, **kwargs)

    @classmethod
    async def verify_encryption_key(
        cls,
        device: BLEDevice,
        key_id: str,
        encryption_key: str,
        model: SwitchbotModel = SwitchbotModel.RELAY_SWITCH_1PM,
        **kwargs: Any,
    ) -> bool:
        return await super().verify_encryption_key(
            device, key_id, encryption_key, model, **kwargs
        )

    def update_from_advertisement(self, advertisement: SwitchBotAdvertisement) -> None:
        """Update device data from advertisement."""
        # Obtain voltage and current through command.
        adv_data = advertisement.data["data"]
        if previous_voltage := self._get_adv_value("voltage"):
            adv_data["voltage"] = previous_voltage
        if previous_current := self._get_adv_value("current"):
            adv_data["current"] = previous_current
        current_state = self._get_adv_value("sequence_number")
        super().update_from_advertisement(advertisement)
        new_state = self._get_adv_value("sequence_number")
        _LOGGER.debug(
            "%s: update advertisement: %s (seq before: %s) (seq after: %s)",
            self.name,
            advertisement,
            current_state,
            new_state,
        )
        if current_state != new_state:
            self._force_next_update = True

    async def update(self, interface: int | None = None) -> None:
        """Update state of device."""
        if info := await self.get_voltage_and_current():
            self._last_full_update = time.monotonic()
            self._update_parsed_data(info)
            self._fire_callbacks()

    async def get_voltage_and_current(self) -> dict[str, Any] | None:
        """Get voltage and current because advtisement don't have these"""
        result = await self._send_command(COMMAND_GET_VOLTAGE_AND_CURRENT)
        ok = self._check_command_result(result, 0, {1})
        if ok:
            return {
                "voltage": ((result[9] << 8) + result[10]) / 10,
                "current": (result[11] << 8) + result[12],
            }
        return None

    async def get_basic_info(self) -> dict[str, Any] | None:
        """Get the current state of the switch."""
        result = await self._send_command(COMMAND_GET_SWITCH_STATE)
        if self._check_command_result(result, 0, {1}):
            return {
                "is_on": result[1] & 0x01 != 0,
            }
        return None

    def poll_needed(self, seconds_since_last_poll: float | None) -> bool:
        """Return if device needs polling."""
        if self._force_next_update:
            self._force_next_update = False
            return True
        if (
            seconds_since_last_poll is not None
            and seconds_since_last_poll < PASSIVE_POLL_INTERVAL
        ):
            return False
        time_since_last_full_update = time.monotonic() - self._last_full_update
        return not time_since_last_full_update < PASSIVE_POLL_INTERVAL

    async def turn_on(self) -> bool:
        """Turn device on."""
        result = await self._send_command(COMMAND_TURN_ON)
        ok = self._check_command_result(result, 0, {1})
        if ok:
            self._override_state({"isOn": True})
            self._fire_callbacks()
        return ok

    async def turn_off(self) -> bool:
        """Turn device off."""
        result = await self._send_command(COMMAND_TURN_OFF)
        ok = self._check_command_result(result, 0, {1})
        if ok:
            self._override_state({"isOn": False})
            self._fire_callbacks()
        return ok

    async def async_toggle(self, **kwargs) -> bool:
        """Toggle device."""
        result = await self._send_command(COMMAND_TOGGLE)
        return self._check_command_result(result, 0, {1})

    def is_on(self) -> bool | None:
        """Return switch state from cache."""
        return self._get_adv_value("isOn")



"""Humidifier adv parser."""

from __future__ import annotations

import logging
from datetime import timedelta

from ..const.evaporative_humidifier import (
    OVER_HUMIDIFY_PROTECTION_MODES,
    TARGET_HUMIDITY_MODES,
    HumidifierMode,
    HumidifierWaterLevel,
)

_LOGGER = logging.getLogger(__name__)

# mfr_data: 943cc68d3d2e
# data: 650000cd802b6300
# data: 650000cd802b6300
# data: 658000c9802b6300


# Low:  658000c5222b6300
# Med:  658000c5432b6300
# High: 658000c5642b6300
def process_wohumidifier(
    data: bytes | None, mfr_data: bytes | None
) -> dict[str, bool | int]:
    """Process WoHumi services data."""
    if data is None:
        return {
            "isOn": None,
            "level": None,
            "switchMode": True,
        }

    return {
        "isOn": bool(data[1]),
        "level": data[4],
        "switchMode": True,
    }


def process_evaporative_humidifier(
    data: bytes | None, mfr_data: bytes | None
) -> dict[str, bool | int]:
    """Process WoHumi services data."""
    if mfr_data is None:
        return {
            "isOn": None,
            "mode": None,
            "target_humidity": None,
            "child_lock": None,
            "over_humidify_protection": None,
            "tank_removed": None,
            "tilted_alert": None,
            "filter_missing": None,
            "humidity": None,
            "temperature": None,
            "filter_run_time": None,
            "filter_alert": None,
            "water_level": None,
        }

    is_on = bool(mfr_data[7] & 0b10000000)
    mode = HumidifierMode(mfr_data[7] & 0b00001111)
    filter_run_time = timedelta(hours=int.from_bytes(mfr_data[12:14], byteorder="big"))
    has_humidity = bool(mfr_data[9] & 0b10000000)
    has_temperature = bool(mfr_data[10] & 0b10000000)
    is_tank_removed = bool(mfr_data[8] & 0b00000100)
    return {
        "isOn": is_on,
        "mode": mode if is_on else None,
        "target_humidity": (mfr_data[16] & 0b01111111)
        if is_on and mode in TARGET_HUMIDITY_MODES
        else None,
        "child_lock": bool(mfr_data[8] & 0b00100000),
        "over_humidify_protection": bool(mfr_data[8] & 0b10000000)
        if is_on and mode in OVER_HUMIDIFY_PROTECTION_MODES
        else None,
        "tank_removed": is_tank_removed,
        "tilted_alert": bool(mfr_data[8] & 0b00000010),
        "filter_missing": bool(mfr_data[8] & 0b00000001),
        "humidity": (mfr_data[9] & 0b01111111) if has_humidity else None,
        "temperature": float(mfr_data[10] & 0b01111111) + float(mfr_data[11] >> 4) / 10
        if has_temperature
        else None,
        "filter_run_time": filter_run_time,
        "filter_alert": filter_run_time.days >= 10,
        "water_level": HumidifierWaterLevel(mfr_data[11] & 0b00000011)
        if not is_tank_removed
        else None,
    }
